#ifndef DAT_H
#define DAT_H

#include <X11/Xlib.h>

#define MAX_BOXED_WINDOWS 16
#define BUTTON_DEFAULT_WIDTH 100
#define NO_ACTIVE_WINDOW -1

struct box;

typedef struct _boxed_window
{
  Window client;
  Window button;
  char * name;
  int number;
  struct box * box;
} BoxedWindow;

typedef struct box
{
  int box_width, box_height;
  Display * dpy;
  GC gc;
  int sn;
  Window win;
  Window box;
  Window bottom;
  BoxedWindow * boxed_windows[MAX_BOXED_WINDOWS];
  int active_window;
  int curtime;
  XFontStruct * font;
} Box;

#define AllButtonMask   (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)
#define ButtonMask (ButtonPressMask|ButtonReleaseMask)
#define MenuMask    (ButtonMask|ButtonMotionMask|ExposureMask)
#define MenuGrabMask    (ButtonMask|ButtonMotionMask|StructureNotifyMask)

int selecting_window;  /* gruik */

#endif
